-------------------------------------------------------------------------------
-- Title      : Tick generator for PWM circuit
-- Project    : 
-------------------------------------------------------------------------------
-- File       : pwmclk.vhdl
-- Author     : <kwalsh@holycross.edu>
-- Company    : 
-- Created    : 2016-02-07
-- Last update: 2016-02-07
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Counts from 0 to 255, and ouputs a 1-tick pulse each iteration.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author      Description
-- 2016-02-07  1.0      kwalsh      Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity pwmclk is

  -- Input clock is 10 mhz, we use upper bits of a 14 bit counter, so
  -- the PWM count does a full round at about 10mhz / 2**14 = 610 hz.

  port (
        clk10                 : in  std_logic;
        rst                   : in  std_logic;
        pwmcnt                : out std_logic_vector(7 downto 0);
        tick                  : out std_logic
  );

end pwmclk;

architecture rtl of pwmclk is

  signal c : std_logic_vector(16 downto 0) := (others => '0');
  signal t : std_logic := '0';

begin

  pwmcnt <= c(16 downto 9);
  tick <= '1' when (c = (c'range => '1')) else '0';

  counter : process (clk10) is
  begin
    if rising_edge(clk10) then
      if rst = '1' then
	c <= (others => '0');
      else
	c <= std_logic_vector(unsigned(c) + 1);
      end if;
    end if;
  end process counter;

end architecture rtl;
