-------------------------------------------------------------------------------
-- Title      : Quad Optical Quadrature Encoder
-- Project    : 
-------------------------------------------------------------------------------
-- File       : quadrature4.vhdl
-- Author     : <kwalsh@holycross.edu>
-- Company    : 
-- Created    : 2016-10-09
-- Last update: 2016-10-09
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Outputs four 32-bit counters based on quadrature encoder inputs.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author      Description
-- 2016-10-09  1.0      kwalsh      Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity quadrature4 is

  port (
        clk                   : in  std_logic;
        rst                   : in  std_logic;
        ba                    : in  std_logic_vector(7 downto 0); -- quadrature (b,a) signal pairs
        q0                    : out  std_logic_vector(31 downto 0); -- counter 0
		  q1                    : out  std_logic_vector(31 downto 0); -- counter 1
		  q2                    : out  std_logic_vector(31 downto 0); -- counter 2
		  q3                    : out  std_logic_vector(31 downto 0)  -- counter 3
  );

end quadrature4;

architecture rtl of quadrature4 is

  type COUNTERS is array (3 downto 0) of std_logic_vector(31 downto 0);
  signal q : COUNTERS;
  
  type QSIGNAL is array (3 downto 0) of std_logic_vector(1 downto 0);
  signal prev : QSIGNAL;
  signal curr : QSIGNAL;
  
  function quad_decode(prev : unsigned; curr : unsigned) return signed is
  begin
    if (prev = 0 and curr = 1) then return to_signed(1, 32);
	 elsif (prev = 1 and curr = 2) then return to_signed(1, 32);
	 elsif (prev = 2 and curr = 3) then return to_signed(1, 32);
	 elsif (prev = 3 and curr = 0) then return to_signed(1, 32);
	 elsif (prev = 3 and curr = 2) then return to_signed(-1, 32);
	 elsif (prev = 2 and curr = 1) then return to_signed(-1, 32);
	 elsif (prev = 1 and curr = 0) then return to_signed(-1, 32);
	 elsif (prev = 0 and curr = 3) then return to_signed(-1, 32);
	 else return to_signed(0, 32);
	 end if;
  end quad_decode;

begin

  q0 <= q(0);
  q1 <= q(1);
  q2 <= q(2);
  q3 <= q(3);
  
  update : process (clk) is
  begin
    if rising_edge(clk) then
	   for i in 0 to 3 loop
        if rst = '1' then
		    prev(i) <= "00";
			 curr(i) <= "00";
		    q(i) <= X"00000000";
		  else
			 prev(i) <= curr(i);
          curr(i) <= ba(2*i+1 downto 2*i);
		    q(i) <= std_logic_vector(signed(q(i)) + quad_decode(unsigned(prev(i)), unsigned(curr(i))));
		  end if;
		end loop;
    end if;
  end process update;

end architecture rtl;
