// Author: kwalsh@holycross.edu

#ifndef JOYSTICK_H
#define JOYSTICK_H

#include <sys/alt_stdio.h>
#include <sys/alt_sys_wrappers.h>
#include <altera_avalon_pio_regs.h>
#include <io.h>
#include <system.h>

// Joystick 0:
//   axis 0 - adc channel 0
//   axis 1 - adc channel 1
//   button 0 - pio_jsb 0
//   button 1 - pio_jsb 1
//   button 2 - pio_jsb 2
//   button 3 - pio_jsb 3
//
// Joystick 1:
//   axis 0 - adc channel 2
//   axis 1 - adc channel 3
//   button 0 - pio_jsb 4
//   button 1 - pio_jsb 5
//   button 2 - pio_jsb 6
//   button 3 - pio_jsb 7

#define ADC_CHAN0  (0x01) // note: Avalon MM Slave docs say offset 0 for read, but lets try offset 1
#define ADC_CHAN1  (0x11) // note: Avalon MM Slave docs say offset 0 for read, but lets try offset 1
#define ADC_CHAN2  (0x21) // note: Avalon MM Slave docs say offset 0 for read, but lets try offset 1
#define ADC_CHAN3  (0x31) // note: Avalon MM Slave docs say offset 0 for read, but lets try offset 1

unsigned int joystick_axis(unsigned int js, unsigned int axis);

#define joystick_buttons() (~(IORD_ALTERA_AVALON_PIO_DATA(PIO_JSB_BASE)))
#define joystick_button(js, b) (joystick_buttons() & (1 << ((((js)*4)) + (b))))

void joystick_calibrate();
void joystick_precalibrated();
int joystick_pos(unsigned int js, unsigned int axis);
int joystick_baxis(unsigned int b_fwd, unsigned int b_rev, int p);

void joystick_demo();


#endif // JOYSTICK_H
