// Author: kwalsh@holycross.edu

#ifndef LCD_H
#define LCD_H

#define LCD_REG_CMD     (0x01)
#define LCD_REG_DATA    (0x11)
#define LCD_REG_STATUS  (0x21) // note: Avalon MM Slave docs say 0x20, but only 0x21 works

#define LCD_SEND_RESET  (0x10)
#define LCD_SEND_CONFIG (0x20)
#define LCD_SEND_READY  (0x30)

#define LCD_XMIT_IDLE       (0x01)
#define LCD_XMIT_SETUP_H    (0x02)
#define LCD_XMIT_ENABLE_H   (0x03)
#define LCD_XMIT_HOLD_H     (0x04)
#define LCD_XMIT_DELAY_H    (0x05)
#define LCD_XMIT_SETUP_L    (0x06)
#define LCD_XMIT_ENABLE_L   (0x07)
#define LCD_XMIT_HOLD_L     (0x08)
#define LCD_XMIT_DELAY_L    (0x09)

#define LCD_IDLE (LCD_SEND_READY|LCD_XMIT_IDLE)

void lcd_out(unsigned int reg, unsigned char data);

#define lcd_cmd(cmd)    lcd_out(LCD_REG_CMD, cmd)
#define lcd_dat(ascii)  lcd_out(LCD_REG_DATA, ascii)
void lcd_putc(char c);
void lcd_puts(const char *str);

// notes on lcd font:
// 0x5c - \\ (backspace) produces yen symbol
// 0x7e - ~ (tilde) produces rightarrow
// 0x7f - \177 (delete) produces leftarrow

// printf-like format strings for lcd, examples:
//  %c - char
//  %s - string
//  %x - hex
//  %3x - 3 hex digits, ignoring upper digits, leading spaces
//  %03x - 3 hex digits, ignoring upper digits, leading zeros
//  \n - go to next line
//  \r - go to start of this line (does not clear the line)
//  \f - clear screen and go home
//  \b - not yet defined
//  \t - write spaces until aligned to multiple of 4
//  \v - move down a line, move back to nearest multiple of 4
//  \aR,C; - (not yet implemented) go to line R, column C (numbers specified in hex, zero-based indexes)
void lcd_printf(const char *fmt, ...);

void lcd_init();
void lcd_clear();
void lcd_goto(unsigned int r, unsigned int c);
#define lcd_home()      lcd_goto(0, 0)

#endif // LCD_H
