// Author: kwalsh@holycross.edu

#include "led.h"
#include "lcd.h"
#include "joystick.h"
#include "motor.h"

// 4 * 0x01 --> bank 0 power
// 4 * 0x11 --> bank 0 en_dir
// 4 * 0x21 --> bank 1 power
// 4 * 0x31 --> bank 1 en_dir
#define PWM_REG(bank, offset) (4*((((bank)*2+(offset))<<4) + 1))

#define PWM_POWER(bank) PWM_REG(bank, 0)
#define PWM_EN_DIR(bank) PWM_REG(bank, 1)

static unsigned int _power[2] = { 0x0, 0x0 };
static unsigned int _en_dir[2] = { 0x0, 0x0 };

void stop_all() {
  IOWR_32DIRECT(PWM_BASE,  PWM_EN_DIR(0), 0);
  IOWR_32DIRECT(PWM_BASE,  PWM_EN_DIR(1), 0);
  IOWR_32DIRECT(PWM_BASE,  PWM_POWER(0), 0);
  IOWR_32DIRECT(PWM_BASE,  PWM_POWER(1), 0);
  _power[0] = 0x0;
  _power[1] = 0x0;
  _en_dir[0] = 0x0;
  _en_dir[1] = 0x0;
}

int motor_en_dir(unsigned int axis, unsigned int enable, unsigned int direction) {
  unsigned int b = 0;
  if (enable)
    b |= 0x2;
  if (direction)
    b |= 0x1;
  unsigned int bank = axis/4;
  unsigned int shift = (axis % 4)*2;
  if ((_en_dir[bank] & (0x3 << shift)) == b)
    return 0;
  _en_dir[bank] &= ~(0x3 << shift);
  _en_dir[bank] |= (b << shift);
  IOWR_32DIRECT(PWM_BASE,  PWM_EN_DIR(bank), _en_dir[bank]);
  return 1;
}

/*
int motor_stability (unsigned int axis, int velocity){
	while (motor_go == true){
		  if (velocity < -255) velocity = -255;
		  else if (velocity > 255) velocity = 255;
		  if (velocity < 0) {
		    int c = motor_speed(axis, velocity);// inverted velocity
		    int d = motor_en_dir(axis, 1, 1);
		    return c || d;
		  } else if (velocity > 0) {
		    int c = motor_speed(axis, -velocity);// inverted velocity
		    int d = motor_en_dir(axis, 1, 0);
		    return c || d;
		  } else {
		    return motor_off(axis);
	}

}
*/

int motor_speed(unsigned int axis, unsigned int speed) {
  speed &= 0xff;
  unsigned int bank = axis/4;
  unsigned int shift = (axis % 4)*8;
  if ((_power[bank] & (0xff << shift)) == speed)
    return 0;
  _power[bank] &= ~(0xff << shift);
  _power[bank] |= (speed << shift);
  IOWR_32DIRECT(PWM_BASE,  PWM_POWER(bank), _power[bank]);
  return 1;
}

int motor_go(unsigned int axis, int velocity) {
  if (velocity < -255) velocity = -255;
  else if (velocity > 255) velocity = 255;
  if (velocity < 0) {
    int c = motor_speed(axis, -velocity);
    int d = motor_en_dir(axis, 1, 1);
    return c || d;
  } else if (velocity > 0) {
    int c = motor_speed(axis, velocity);
    int d = motor_en_dir(axis, 1, 0);
    return c || d;
  } else {
    return motor_off(axis);
  }
}


void motor_demo() {
  lcd_printf("buttons 1 & 2 go\n");
  lcd_printf("buttons 3 & 4 pwr\n");
  motor_speed(7, 0x80);

  while (1) {
    if (joystick_button(0, 3)) {
      motor_speed(7, 0xc0);
    } else if (joystick_button(0, 4)) {
      motor_speed(7, 0xff);
    }
    if (joystick_button(0, 2)) {
      if (motor_on(7, 0)) {
	lcd_printf("\rleft ");
      }
    } else if (joystick_button(0, 1)) {
      if (motor_on(7, 1)) {
	lcd_printf("\rright");
      }
    } else {
      if (motor_off(7)) {
	lcd_printf("\rstop");
      }
    }
  }
}

void motor_demo2() {
  lcd_printf("  button 6: m++\n");
  lcd_printf("  button 7: m--\n");
  lcd_printf("joystick 0: pwr\n");
  unsigned int m = 0;
  while (1) {
    if (joystick_button(0, 6)) {
      while (joystick_button(0, 6));
      if (m < 7)
	m++;
    } else if (joystick_button(0, 7)) {
      while (joystick_button(0, 7));
      if (m > 0)
	m--;
    }
    int p = joystick_pos(0, 0);
    motor_go(m, p);
    lcd_printf("\rmotor %x: %c%2x ", m, (p < 0 ? '-' : (p > 0 ? '+' : ' ')), (p < 0 ? -p : p));
    usleep(50000);
  }
}
